package cz.muni.fi.icehockeymanager.arenareservation.facade;

import cz.muni.fi.icehockeymanager.arenareservation.api.edit.EditArenaReservationRecordDto;
import cz.muni.fi.icehockeymanager.arenareservation.api.view.ArenaReservationRecordDetailDto;
import cz.muni.fi.icehockeymanager.arenareservation.api.create.CreateArenaReservationRecordDto;
import cz.muni.fi.icehockeymanager.arenareservation.api.view.ArenaReservationRecordDto;
import cz.muni.fi.icehockeymanager.arenareservation.data.domain.ReservationRecord;
import cz.muni.fi.icehockeymanager.arenareservation.exceptions.TimeTableCollisionException;
import cz.muni.fi.icehockeymanager.arenareservation.mappers.ReservationRecordMapper;
import cz.muni.fi.icehockeymanager.arenareservation.service.ArenaService;
import cz.muni.fi.icehockeymanager.arenareservation.service.ContactPersonService;
import cz.muni.fi.icehockeymanager.arenareservation.service.ReservationService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.sql.Timestamp;
import java.time.LocalDateTime;
import java.time.Month;
import java.util.List;

@Service
@Transactional
public class ReservationFacade {

    private final ReservationService reservationService;
    private final ArenaService arenaService;
    private final ContactPersonService contactPersonService;
    private final ReservationRecordMapper reservationRecordMapper;

    @Autowired
    public ReservationFacade(ReservationService reservationService, ArenaService arenaService,
                             ContactPersonService contactPersonService,
                             ReservationRecordMapper reservationRecordMapper) {
        this.reservationService = reservationService;
        this.arenaService = arenaService;
        this.contactPersonService = contactPersonService;
        this.reservationRecordMapper = reservationRecordMapper;
    }

    private Timestamp convertFromLocalDateTime(LocalDateTime source, boolean minFlag) {
        return Timestamp.valueOf(
                        source == null ?
                                (minFlag ? LocalDateTime.of(1970, Month.JANUARY, 1, 0, 0)
                                        : LocalDateTime.of(9999, Month.JANUARY, 1, 0, 0))
                                : source);
    }
    public List<ArenaReservationRecordDto> getTimetableForArena(
            Long arenaID, LocalDateTime fromDateTime, LocalDateTime toDateTime) {
        Timestamp fromTimestamp = convertFromLocalDateTime(fromDateTime, true);
        Timestamp toTimestamp = convertFromLocalDateTime(toDateTime, false);

        return reservationRecordMapper.mapReservationRecordListToBasicReservationRecordDtoList(
                reservationService.getAllRecordsByArenaId(arenaID, fromTimestamp, toTimestamp));
    }

    public List<ArenaReservationRecordDetailDto> getReservationsByPersonId(
            Long personId, LocalDateTime fromDateTime, LocalDateTime toDateTime) {
        Timestamp fromTimestamp = convertFromLocalDateTime(fromDateTime, true);
        Timestamp toTimestamp = convertFromLocalDateTime(toDateTime, false);

        return reservationRecordMapper.mapReservationRecordListToRecordDtoList(
                reservationService.getAllRecordsByUserId(personId, fromTimestamp, toTimestamp));
    }

    public ArenaReservationRecordDetailDto createNewReservation(Long arenaId, CreateArenaReservationRecordDto record) {
        ReservationRecord reservationRecord = reservationRecordMapper.mapCreateReservationDtoToReservationRecord(record);

        if (reservationService.checkTimeTableCollision(arenaId, reservationRecord.getDateTimeFrom(),
                reservationRecord.getDateTimeTo()))
            throw new TimeTableCollisionException(TimeTableExceptionMessage);

        reservationRecord.setArena(arenaService.findArenaById(arenaId));
        reservationRecord.setContactPerson(contactPersonService.findById(record.getContactPersonId()));
        return reservationRecordMapper.mapReservationRecordToRecordDto(
                reservationService.saveReservationRecord(reservationRecord));
    }


    public ArenaReservationRecordDetailDto modifyReservation(Long recordId,
                                                             EditArenaReservationRecordDto editArenaReservationRecordDto) {
        ReservationRecord reservationRecord = reservationService.findById(recordId);

        if (reservationService.checkTimeTableCollision(reservationRecord.getArena().getId(),
                reservationRecord.getDateTimeFrom(),
                reservationRecord.getDateTimeTo()))
            throw new TimeTableCollisionException(TimeTableExceptionMessage);

        if (editArenaReservationRecordDto.getReservationType() != null)
            reservationRecord.setReservationType(editArenaReservationRecordDto.getReservationType());
        if (editArenaReservationRecordDto.getDateTimeFrom() != null)
            reservationRecord.setDateTimeFrom(editArenaReservationRecordDto.getDateTimeFrom());
        if (editArenaReservationRecordDto.getDateTimeTo() != null)
            reservationRecord.setDateTimeTo(editArenaReservationRecordDto.getDateTimeTo());

        return reservationRecordMapper.mapReservationRecordToRecordDto(
                reservationService.saveReservationRecord(reservationRecord));
    }

    public void deleteReservation(Long recordID) {
        reservationService.deleteRecord(recordID);
    }

    private static String TimeTableExceptionMessage = "Specified Time will create Conflict with other Reservations";
}
