package cz.muni.fi.icehockeymanager.arenareservation.rest;

import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import cz.muni.fi.icehockeymanager.arenareservation.api.edit.EditArenaDto;
import cz.muni.fi.icehockeymanager.arenareservation.api.view.AddressDto;
import cz.muni.fi.icehockeymanager.arenareservation.api.view.ArenaDto;
import cz.muni.fi.icehockeymanager.arenareservation.api.view.ArenaDetailDto;
import cz.muni.fi.icehockeymanager.arenareservation.api.create.CreateArenaDto;
import cz.muni.fi.icehockeymanager.arenareservation.facade.ArenaFacade;
import org.junit.jupiter.api.Test;
import org.mockito.Mockito;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.WebMvcTest;
import org.springframework.boot.test.mock.mockito.MockBean;
import org.springframework.http.MediaType;
import org.springframework.test.web.servlet.MockMvc;

import static org.mockito.ArgumentMatchers.any;
import static org.mockito.BDDMockito.given;
import static org.hamcrest.MatcherAssert.assertThat;
import static org.hamcrest.Matchers.equalTo;
import static org.hamcrest.Matchers.is;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.*;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;

import java.nio.charset.StandardCharsets;
import java.util.ArrayList;
import java.util.List;

@WebMvcTest(ArenaRestController.class)
public class ArenaRestControllerUnitTests {

    private static final Logger log = LoggerFactory.getLogger(ArenaRestControllerUnitTests.class);

    private static List<ArenaDetailDto> arenaList = new ArrayList<>();
    @Autowired
    private MockMvc mockMvc;

    @MockBean
    private ArenaFacade mockArenaFacade;
    @Autowired
    private ObjectMapper objectMapper;

    private static ArenaDetailDto createDto(Long id, String name, String street, String number,
                                            String city, String region, String country) {
        ArenaDetailDto arenaDto = new ArenaDetailDto();
        arenaDto.setId(id);
        arenaDto.setArenaName(name);

        AddressDto addressDto = new AddressDto();
        addressDto.setId(id);
        addressDto.setStreet(street);
        addressDto.setStreetNumber(number);
        addressDto.setCity(city);
        addressDto.setRegion(region);
        addressDto.setCountryISO(country);
        arenaDto.setAddressDto(addressDto);

        return arenaDto;
    }

    private boolean compareArenaDtos(ArenaDto first, ArenaDto second) {
        return (
                first.getId().equals(second.getId()) &&
                first.getArenaName().equals(second.getArenaName()) &&
                first.getCity().equals(second.getCity()) &&
                first.getCountryISO().equals(second.getCountryISO())
                );
    }

    @Test
    public void testGetArenas() throws Exception {
        log.debug("Performing Unit Test for getting Arenas");

        arenaList.add(createDto(0L, "O2 Arena", "Ocelářská", "460/2", "Prague",
                "Prague 9", "CZE"));
        arenaList.add(createDto(1L, "Lanxess Arena", "Willy-Brandt-Platz", "3", "Cologne",
                "North Rhine-Westphalia", "DEU"));
        arenaList.add(createDto(2L, "PostFinance Arena", "Mingerstrasse", "12", "Bern",
                "Bern", "CHE"));
        arenaList.add(createDto(3L, "Mercedes-Benz Arena", "Mercedes-Platz", "1", "Berlin",
                "Berlin-Brandenburg capital", "DEU"));

        List<ArenaDto> listArenaDto = new ArrayList<>();
        for (ArenaDetailDto arenaDetailDto : arenaList) {
            ArenaDto arenaDto = new ArenaDto();
            arenaDto.setId(arenaDetailDto.getId());
            arenaDto.setArenaName(arenaDetailDto.getArenaName());
            arenaDto.setCity(arenaDetailDto.getAddressDto().getCity());
            arenaDto.setCountryISO(arenaDetailDto.getAddressDto().getCountryISO());
            listArenaDto.add(arenaDto);
        }

        given(mockArenaFacade.getAllArenas()).willReturn(listArenaDto);

        String result = mockMvc.perform(get("/arenas").contentType(MediaType.APPLICATION_JSON))
                .andExpect(status().isOk()).andReturn().getResponse().getContentAsString(StandardCharsets.UTF_8);

        List<ArenaDto> restList = objectMapper.readValue(result, new TypeReference<List<ArenaDto>>(){});

        assertThat("Size", listArenaDto.size(), is(equalTo(restList.size())));
        for (int index = 0; index < listArenaDto.size(); index++) {
            assertThat(compareArenaDtos(listArenaDto.get(index), restList.get(index)), is(true));
        }
    }
    
    @Test
    public void testGetArena() throws Exception {
        log.debug("Performing Unit Test for get specific Arena");
        
        ArenaDetailDto arenaDetailDto = createDto(0L, "O2 Arena", "Ocelářská", "460/2", "Prague",
                "Prague 9", "CZE");
        
        given(mockArenaFacade.getArenaDetailView(0L)).willReturn(arenaDetailDto);

        String result = mockMvc.perform(get("/arenas/" + arenaDetailDto.getId())
                        .contentType(MediaType.APPLICATION_JSON))
                .andExpect(status().isOk()).andReturn().getResponse().getContentAsString(StandardCharsets.UTF_8);

        assertThat(result.equals(objectMapper.writeValueAsString(arenaDetailDto)), is(true));
    }

    @Test
    public void testAddArena() throws Exception {
        log.debug("Performing Unit Test for adding new Arena");

        ArenaDetailDto arenaDto = createDto(0L, "Mercedes-Benz Arena", "Mercedes-Platz", "1", "Berlin",
                "Berlin-Brandenburg capital", "DEU");

        CreateArenaDto createArenaDto = new CreateArenaDto();
        createArenaDto.setArenaName(arenaDto.getArenaName());
        createArenaDto.setStreet(arenaDto.getAddressDto().getStreet());
        createArenaDto.setStreetNumber(arenaDto.getAddressDto().getStreetNumber());
        createArenaDto.setCity(arenaDto.getAddressDto().getCity());
        createArenaDto.setRegion(arenaDto.getAddressDto().getRegion());
        createArenaDto.setCountryISO(arenaDto.getAddressDto().getCountryISO());

        Mockito.when(mockArenaFacade.createArena(any(CreateArenaDto.class))).thenReturn(arenaDto);

        String result = mockMvc.perform(post("/arenas")
                        .contentType(MediaType.APPLICATION_JSON)
                        .content(objectMapper.writeValueAsString(createArenaDto))
                )
                .andExpect(status().isCreated())
                .andReturn().getResponse().getContentAsString(StandardCharsets.UTF_8);

        assertThat(result.equals(objectMapper.writeValueAsString(arenaDto)), is(true));
    }

    @Test
    public void testAddEmptyArena() throws Exception {
        log.debug("Performing Unit Test for adding empty Arena record");

        mockMvc.perform(post("/arenas")
                .contentType(MediaType.APPLICATION_JSON)
                )
                .andExpect(status().is4xxClientError());
    }

    @Test
    public void testModifyArena() throws Exception {
        log.debug("Performing Unit Test for modifying Arena record");
        testAddArena();

        ArenaDetailDto arenaDto = createDto(0L, "Mercedes Benz Arena", "Mercedes-Platz", "1", "Berlin",
                "Berlin-Brandenburg capital", "DEU");
        arenaDto.setArenaName("Mercedes Benz Arena 200");

        EditArenaDto editArenaDto = new EditArenaDto();
        editArenaDto.setArenaName("Mercedes Benz Arena");
        editArenaDto.setCapacity(arenaDto.getCapacity());

        Mockito.when(mockArenaFacade.modifyArena(any(Long.class), any(EditArenaDto.class))).thenReturn(arenaDto);

        String result = mockMvc.perform(patch("/arenas/" + arenaDto.getId())
                        .contentType(MediaType.APPLICATION_JSON)
                        .content(objectMapper.writeValueAsString(arenaDto))
                )
                .andExpect(status().isOk())
                .andReturn().getResponse().getContentAsString(StandardCharsets.UTF_8);

        assertThat(result.equals(objectMapper.writeValueAsString(arenaDto)), is(true));
    }


    @Test
    public void testDeleteArena() throws  Exception {
        log.debug("Performing Unit Test for deleting Arena record");

        mockMvc.perform(delete("/arenas/0"))
                .andExpect(status().isNoContent());
        Mockito.verify(mockArenaFacade).deleteArena(0L);
    }

    @Test
    public void testMediaTypeNotSupported() throws Exception {
        log.debug("Performing Unit Test for not supported media type");
        CreateArenaDto createArenaDto = new CreateArenaDto();

        mockMvc.perform(post("/arenas")
                .contentType(MediaType.APPLICATION_PDF)
                .content(objectMapper.writeValueAsString(createArenaDto))
        ).andExpect(status().is4xxClientError());
    }

    @Test
    public void testWrongUrl() throws Exception {
        log.debug("Performing Unit Test for incorrect URL");
        mockMvc.perform(post("/arenas1")
                .contentType(MediaType.APPLICATION_JSON)
                .content(objectMapper.writeValueAsString(new CreateArenaDto()))
        ).andExpect(status().is4xxClientError());
    }

    @Test
    public void testHttpGetWithWrongParameters() throws Exception {
        log.debug("Performing Unit Test for Get HTTP method with wrong parameters");
        mockMvc.perform(get("/arenas/arenas")
                       .contentType(MediaType.APPLICATION_JSON)
               ).andExpect(status().is4xxClientError());
    }

    @Test
    public void testHttpDeleteWithWrongParameters() throws Exception {
        log.debug("Performing Unit Test for Delete HTTP method with wrong parameters");
        mockMvc.perform(delete("/arenas/arenas")
                .contentType(MediaType.APPLICATION_JSON)
        ).andExpect(status().is4xxClientError());
    }

    @Test
    public void testNonSupportedHttpMethodPut() throws Exception {
        log.debug("Performing Unit Test for not supported HTTP method");
        mockMvc.perform(put("/arenas/1")
                .contentType(MediaType.APPLICATION_JSON)
                .content(objectMapper.writeValueAsString(new ArenaDetailDto()))
        ).andExpect(status().is4xxClientError());
    }

    @Test
    public void testServerSideError() throws Exception {
        log.debug("Performing Unit Test for Server-side error");

        Mockito.when(mockArenaFacade.createArena(any()))
                .thenThrow(new NullPointerException());

        mockMvc.perform(post("/arenas")
                .contentType(MediaType.APPLICATION_JSON)
                .content(objectMapper.writeValueAsString(new CreateArenaDto()))
        ).andExpect(status().is5xxServerError());
    }
}
