package cz.muni.fi.icehockeymanager.arenareservation.rest;

import com.fasterxml.jackson.databind.ObjectMapper;
import cz.muni.fi.icehockeymanager.arenareservation.api.edit.EditArenaReservationRecordDto;
import cz.muni.fi.icehockeymanager.arenareservation.api.view.*;
import cz.muni.fi.icehockeymanager.arenareservation.api.create.CreateArenaReservationRecordDto;
import cz.muni.fi.icehockeymanager.arenareservation.data.enums.ReservationType;
import cz.muni.fi.icehockeymanager.arenareservation.facade.ReservationFacade;
import org.junit.jupiter.api.Test;
import org.mockito.Mockito;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.WebMvcTest;
import org.springframework.boot.test.mock.mockito.MockBean;
import org.springframework.http.MediaType;
import org.springframework.test.web.servlet.MockMvc;
import java.nio.charset.StandardCharsets;
import java.sql.Timestamp;
import java.time.LocalDateTime;
import java.time.Month;
import java.util.ArrayList;
import java.util.List;

import static org.hamcrest.MatcherAssert.assertThat;
import static org.hamcrest.Matchers.is;
import static org.mockito.ArgumentMatchers.any;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.*;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.post;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.put;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;

@WebMvcTest(ReservationRestController.class)
public class ReservationRestControllerUniTests {

    private static final Logger log = LoggerFactory.getLogger(ArenaRestControllerUnitTests.class);
    @Autowired
    private MockMvc mockMvc;
    @MockBean
    private ReservationFacade mockReservationFacade;
    @Autowired
    private ObjectMapper objectMapper;

    @Test
    public void testAddNewReservation() throws Exception {
        log.debug("Performing Unit Test for adding new Reservation");

        ArenaReservationRecordDetailDto arenaReservationRecordDetailDto = new ArenaReservationRecordDetailDto();
        arenaReservationRecordDetailDto.setId(0L);
        arenaReservationRecordDetailDto.setArenaDetailDto(new ArenaDetailDto());
        arenaReservationRecordDetailDto.setContactPersonDetailDto(new ContactPersonDetailDto());
        arenaReservationRecordDetailDto.setDateTimeTo(
                Timestamp.valueOf(LocalDateTime.of(2026, Month.JANUARY, 1, 0, 0)));
        arenaReservationRecordDetailDto.setDateTimeFrom(
                Timestamp.valueOf(LocalDateTime.of(2025, Month.JANUARY, 1, 0, 0)));


        Mockito.when(mockReservationFacade.createNewReservation(any(Long.class), any(CreateArenaReservationRecordDto.class)))
                .thenReturn(arenaReservationRecordDetailDto);

        String result = mockMvc.perform(post("/reservations/1")
                        .contentType(MediaType.APPLICATION_JSON)
                        .content(objectMapper.writeValueAsString(new CreateArenaReservationRecordDto()))
                )
                .andExpect(status().isCreated())
                .andReturn().getResponse().getContentAsString(StandardCharsets.UTF_8);

        assertThat(result.equals(objectMapper.writeValueAsString(arenaReservationRecordDetailDto)), is(true));
    }

    @Test
    public void testModifyReservation() throws Exception {
        log.debug("Performing Unit Test for modifying existing Reservation");

        ArenaReservationRecordDetailDto arenaReservationRecordDetailDto = new ArenaReservationRecordDetailDto();
        arenaReservationRecordDetailDto.setId(1L);
        arenaReservationRecordDetailDto.setArenaDetailDto(new ArenaDetailDto());
        arenaReservationRecordDetailDto.setContactPersonDetailDto(new ContactPersonDetailDto());
        arenaReservationRecordDetailDto.setDateTimeTo(Timestamp.valueOf(LocalDateTime.now().plusHours(10)));
        arenaReservationRecordDetailDto.setDateTimeFrom(Timestamp.valueOf(LocalDateTime.now()));
        arenaReservationRecordDetailDto.setReservationType(ReservationType.LEAGUE_MATCH);

        EditArenaReservationRecordDto editArenaReservationRecordDto = new EditArenaReservationRecordDto();
        editArenaReservationRecordDto.setReservationType(ReservationType.LEAGUE_MATCH);


        Mockito.when(mockReservationFacade.modifyReservation(any(Long.class), any(EditArenaReservationRecordDto.class)))
                .thenReturn(arenaReservationRecordDetailDto);

        String result = mockMvc.perform(patch("/reservations/" + arenaReservationRecordDetailDto.getId())
                    .contentType(MediaType.APPLICATION_JSON)
                    .content(objectMapper.writeValueAsString(editArenaReservationRecordDto))
                )
                .andExpect(status().isOk())
                .andReturn().getResponse().getContentAsString(StandardCharsets.UTF_8);

        assertThat(result.equals(objectMapper.writeValueAsString(arenaReservationRecordDetailDto)), is(true));
    }

    @Test
    public void testGetTimeTable() throws Exception {
        log.debug("Performing Unit Test for getting reservation Timetable for specific Arena");

        List<ArenaReservationRecordDto> recordList = new ArrayList<>();

        ArenaReservationRecordDto arenaReservationRecordDto = new ArenaReservationRecordDto();
        arenaReservationRecordDto.setId(2L);
        arenaReservationRecordDto.setArenaDto(new ArenaDto());
        arenaReservationRecordDto.setDateTimeTo(Timestamp.valueOf(LocalDateTime.now().plusHours(10)));
        arenaReservationRecordDto.setDateTimeFrom(Timestamp.valueOf(LocalDateTime.now()));
        recordList.add(arenaReservationRecordDto);

        arenaReservationRecordDto = new ArenaReservationRecordDto();
        arenaReservationRecordDto.setId(3L);
        arenaReservationRecordDto.setArenaDto(new ArenaDto());
        arenaReservationRecordDto.setDateTimeTo(Timestamp.valueOf(LocalDateTime.now().plusHours(10)));
        arenaReservationRecordDto.setDateTimeFrom(Timestamp.valueOf(LocalDateTime.now()));
        recordList.add(arenaReservationRecordDto);

        Mockito.when(mockReservationFacade.getTimetableForArena(any(Long.class),
                        any(LocalDateTime.class), any(LocalDateTime.class)))
                .thenReturn(recordList);


        String result = mockMvc.perform(
                get("/reservations/arenas/0?from=2023-03-26T14:44:29&to=2023-03-26T14:44:29")
                    .contentType(MediaType.APPLICATION_JSON)
                )
                .andExpect(status().isOk())
                .andReturn().getResponse().getContentAsString(StandardCharsets.UTF_8);

        assertThat(result.equals(objectMapper.writeValueAsString(recordList)), is(true));
    }

    @Test
    public void testUserSpecificReservations() throws Exception{
        log.debug("Performing Unit Test for getting reservations for specific Contact Person");

        List<ArenaReservationRecordDetailDto> recordList = new ArrayList<>();

        ArenaReservationRecordDetailDto arenaReservationRecordDetailDto = new ArenaReservationRecordDetailDto();
        arenaReservationRecordDetailDto.setId(2L);
        arenaReservationRecordDetailDto.setArenaDetailDto(new ArenaDetailDto());
        arenaReservationRecordDetailDto.setContactPersonDetailDto(new ContactPersonDetailDto());
        arenaReservationRecordDetailDto.setDateTimeTo(Timestamp.valueOf(LocalDateTime.now().plusHours(10)));
        arenaReservationRecordDetailDto.setDateTimeFrom(Timestamp.valueOf(LocalDateTime.now()));
        recordList.add(arenaReservationRecordDetailDto);

        arenaReservationRecordDetailDto = new ArenaReservationRecordDetailDto();
        arenaReservationRecordDetailDto.setId(3L);
        arenaReservationRecordDetailDto.setArenaDetailDto(new ArenaDetailDto());
        arenaReservationRecordDetailDto.setContactPersonDetailDto(new ContactPersonDetailDto());
        arenaReservationRecordDetailDto.setDateTimeTo(Timestamp.valueOf(LocalDateTime.now().plusHours(10)));
        arenaReservationRecordDetailDto.setDateTimeFrom(Timestamp.valueOf(LocalDateTime.now()));
        recordList.add(arenaReservationRecordDetailDto);

        Mockito.when(mockReservationFacade.getReservationsByPersonId(any(Long.class),
                        any(LocalDateTime.class), any(LocalDateTime.class)))
                .thenReturn(recordList);


        String result = mockMvc.perform(
                        get("/reservations/0?from=2023-03-26T14:44:29&to=2023-03-26T14:44:29")
                                .contentType(MediaType.APPLICATION_JSON)
                )
                .andExpect(status().isOk())
                .andReturn().getResponse().getContentAsString(StandardCharsets.UTF_8);

        assertThat(result.equals(objectMapper.writeValueAsString(recordList)), is(true));
    }


    @Test
    public void testDeleteReservation() throws Exception {
        log.debug("Performing Unit Test for deleting/canceling existing Reservation");

        mockMvc.perform(delete("/reservations/0"))
                    .andExpect(status().isNoContent());

        Mockito.verify(mockReservationFacade).deleteReservation(0L);
    }

    @Test
    public void testMediaTypeNotSupported() throws Exception {
        log.debug("Performing Unit Test for not supported media type");

        mockMvc.perform(post("/reservations")
                .contentType(MediaType.APPLICATION_PDF)
                .content(objectMapper.writeValueAsString(new CreateArenaReservationRecordDto()))
        ).andExpect(status().is4xxClientError());
    }

    @Test
    public void testWrongUrl() throws Exception {
        log.debug("Performing Unit Test for incorrect URL");
        String result =
        mockMvc.perform(post("/reservations123")
                .contentType(MediaType.APPLICATION_JSON)
                .content(objectMapper.writeValueAsString(new CreateArenaReservationRecordDto()))
        ).andExpect(status().is4xxClientError())
                .andReturn().getResponse().getContentAsString();
    }

    @Test
    public void testHttpGetWithWrongParameters() throws Exception {
        log.debug("Performing Unit Test for Get HTTP method with wrong parameters");
        mockMvc.perform(get("/reservations/first")
                .contentType(MediaType.APPLICATION_JSON)
        ).andExpect(status().is4xxClientError());
    }

    @Test
    public void testHttpDeleteWithWrongParameters() throws Exception {
        log.debug("Performing Unit Test for Delete HTTP method with wrong parameters");
        mockMvc.perform(delete("/reservations/second")
                .contentType(MediaType.APPLICATION_JSON)
        ).andExpect(status().is4xxClientError());
    }

    @Test
    public void testNonSupportedHttpMethodPut() throws Exception {
        log.debug("Performing Unit Test for not supported HTTP method");
        mockMvc.perform(put("/reservations/1")
                .contentType(MediaType.APPLICATION_JSON)
                .content(objectMapper.writeValueAsString(new ArenaReservationRecordDetailDto()))
        ).andExpect(status().is4xxClientError());
    }

    @Test
    public void testServerSideError() throws Exception {
        log.debug("Performing Unit Test for Server-side error");

        Mockito.when(mockReservationFacade.createNewReservation(any(), any()))
                .thenThrow(new NullPointerException());

        mockMvc.perform(post("/reservations/1")
                .contentType(MediaType.APPLICATION_JSON)
                .content(objectMapper.writeValueAsString(new CreateArenaReservationRecordDto()))
        ).andExpect(status().is5xxServerError());
    }
}
