package cz.muni.fi.icehockeymanager.arenareservation.service;

import cz.muni.fi.icehockeymanager.arenareservation.data.domain.Address;
import cz.muni.fi.icehockeymanager.arenareservation.data.domain.Arena;
import cz.muni.fi.icehockeymanager.arenareservation.data.domain.ContactPerson;
import cz.muni.fi.icehockeymanager.arenareservation.data.domain.ReservationRecord;
import cz.muni.fi.icehockeymanager.arenareservation.data.enums.ReservationType;
import cz.muni.fi.icehockeymanager.arenareservation.data.repository.ReservationRepository;
import cz.muni.fi.icehockeymanager.arenareservation.exceptions.ResourceNotFoundException;
import org.junit.jupiter.api.BeforeAll;
import org.junit.jupiter.api.Test;
import org.mockito.Mockito;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.boot.test.mock.mockito.MockBean;

import java.sql.Timestamp;
import java.time.LocalDateTime;
import java.time.Month;
import java.util.*;

import static org.junit.Assert.assertThrows;
import static org.mockito.ArgumentMatchers.any;

@SpringBootTest
public class ArenaReservationServiceTests {
    @Autowired
    private ReservationService reservationService;

    @MockBean
    private ReservationRepository reservationRepository;

    private static List<ContactPerson> contactPersonList = Arrays.asList(
        new ContactPerson(1L, "John", "Wick", "john.wick123@example.com", "+1234252", new HashSet<>()),
        new ContactPerson(2L, "Rick", "Astley", "rick1919@example.com", "+09412 412", new HashSet<>())
    );

    private static List<Arena> arenas = new ArrayList<>();

    private static List<ReservationRecord> reservationRecords = new ArrayList<>();

    @BeforeAll
    public static void initBeforeTests() {
        Arena arenaOne = new Arena(1L, new Address(), "O2 Brno Arena",
                2023, 10200, 12343, new HashSet<>());
        Address arenaOneAddress = new Address(1L, "Botanická", "10/20",
                "Brno", "Moravia", "CZE", arenaOne);
        arenaOne.setAddress(arenaOneAddress);
        arenas.add(arenaOne);

        Arena arenaTwo = new Arena(2L, new Address(), "Mercedes-Benz Arena",
                2015, 8400, 12343, new HashSet<>());
        Address arenaTwoAddress = new Address(2L, "Benz", "100A/23II",
                "Berlin", "Berlin", "DEU", arenaTwo);
        arenaTwo.setAddress(arenaTwoAddress);
        arenas.add(arenaTwo);

        reservationRecords.add(new ReservationRecord(1L, arenaOne, contactPersonList.get(0),
                Timestamp.valueOf(LocalDateTime.of(2026, Month.JANUARY, 1, 10, 0)),
                Timestamp.valueOf(LocalDateTime.of(2026, Month.JANUARY, 1, 18, 0)),
                ReservationType.LEAGUE_MATCH));
        reservationRecords.add(new ReservationRecord(2L, arenaOne, contactPersonList.get(0),
                Timestamp.valueOf(LocalDateTime.of(2026, Month.JANUARY, 2, 10, 0)),
                Timestamp.valueOf(LocalDateTime.of(2026, Month.JANUARY, 2, 18, 0)),
                ReservationType.LEAGUE_MATCH));
        reservationRecords.add(new ReservationRecord(3L, arenaOne, contactPersonList.get(0),
                Timestamp.valueOf(LocalDateTime.of(2026, Month.JANUARY, 10, 10, 0)),
                Timestamp.valueOf(LocalDateTime.of(2026, Month.JANUARY, 10, 18, 0)),
                ReservationType.LEAGUE_MATCH));
        reservationRecords.add(new ReservationRecord(4L, arenaTwo, contactPersonList.get(0),
                Timestamp.valueOf(LocalDateTime.of(2026, Month.JANUARY, 22, 10, 0)),
                Timestamp.valueOf(LocalDateTime.of(2026, Month.JANUARY, 22, 22, 0)),
                ReservationType.LEAGUE_MATCH));

        reservationRecords.add(new ReservationRecord(5L, arenaTwo, contactPersonList.get(1),
                Timestamp.valueOf(LocalDateTime.of(2026, Month.FEBRUARY, 1, 10, 0)),
                Timestamp.valueOf(LocalDateTime.of(2026, Month.FEBRUARY, 1, 18, 0)),
                ReservationType.TRAINING));
        reservationRecords.add(new ReservationRecord(6L, arenaTwo, contactPersonList.get(1),
                Timestamp.valueOf(LocalDateTime.of(2026, Month.FEBRUARY, 2, 10, 0)),
                Timestamp.valueOf(LocalDateTime.of(2026, Month.FEBRUARY, 2, 18, 0)),
                ReservationType.TRAINING));
        reservationRecords.add(new ReservationRecord(7L, arenaTwo, contactPersonList.get(1),
                Timestamp.valueOf(LocalDateTime.of(2026, Month.FEBRUARY, 10, 10, 0)),
                Timestamp.valueOf(LocalDateTime.of(2026, Month.FEBRUARY, 10, 18, 0)),
                ReservationType.OTHER));
        reservationRecords.add(new ReservationRecord(8L, arenaTwo, contactPersonList.get(1),
                Timestamp.valueOf(LocalDateTime.of(2026, Month.FEBRUARY, 22, 10, 0)),
                Timestamp.valueOf(LocalDateTime.of(2026, Month.FEBRUARY, 22, 22, 0)),
                ReservationType.FRIENDLY_MATCH));
    }

    @Test
    public void testCreateReservation() {
        Mockito.when(reservationRepository.save(any(ReservationRecord.class)))
                .thenReturn(reservationRecords.get(0));
        ReservationRecord record = reservationService.saveReservationRecord(reservationRecords.get(0));
        assert record.equals(reservationRecords.get(0));
    }

    @Test
    public void testGetAllReservationsByUserId() {
        Long userId = contactPersonList.get(0).getId();
        List<ReservationRecord> userRecords = new ArrayList<>(reservationRecords);
        userRecords.removeIf(reservationRecord ->
                !Objects.equals(reservationRecord.getContactPerson().getId(), userId));

        Mockito.when(reservationRepository.findAllWithUserIdAndDateFiltering(
                        userId, Timestamp.valueOf(LocalDateTime.MIN), Timestamp.valueOf(LocalDateTime.MAX)))
                .thenReturn(userRecords);

        List<ReservationRecord> records = reservationService.getAllRecordsByUserId(userId,
                Timestamp.valueOf(LocalDateTime.MIN), Timestamp.valueOf(LocalDateTime.MAX));
        assert records.size() == userRecords.size();

        for (int index = 0; index < userRecords.size(); index++) {
            assert records.get(index).equals(userRecords.get(index));
        }
    }

    @Test
    public void testGetAllReservationsByUserIdEmpty() {
        Mockito.when(reservationRepository.findAllWithUserIdAndDateFiltering(
                any(Long.class), any(Timestamp.class), any(Timestamp.class)))
                .thenReturn(new ArrayList<>());

        List<ReservationRecord> records = reservationService.getAllRecordsByUserId(123L,
                Timestamp.valueOf(LocalDateTime.MIN), Timestamp.valueOf(LocalDateTime.MAX));
        assert records.size() == 0;
    }

    @Test
    public void testGetAllReservationsByArenaId() {
        Long arenaId = arenas.get(0).getId();
        List<ReservationRecord> arenaRecords = new ArrayList<>(reservationRecords);
        arenaRecords.removeIf(reservationRecord ->
                !Objects.equals(reservationRecord.getArena().getId(), arenaId));

        Mockito.when(reservationRepository.findAllByArenaIdWithDateFiltering(
                        arenaId, Timestamp.valueOf(LocalDateTime.MIN), Timestamp.valueOf(LocalDateTime.MAX)))
                .thenReturn(arenaRecords);

        List<ReservationRecord> records = reservationService.getAllRecordsByArenaId(arenaId,
                        Timestamp.valueOf(LocalDateTime.MIN), Timestamp.valueOf(LocalDateTime.MAX));
        assert records.size() == arenaRecords.size();

        for (int index = 0; index < arenaRecords.size(); index++) {
            assert records.get(index).equals(arenaRecords.get(index));
        }
    }

    @Test
    public void testGetAllReservationsByArenaIdEmpty() {
        Mockito.when(reservationRepository.findAllByArenaIdWithDateFiltering(
                        any(Long.class), any(Timestamp.class), any(Timestamp.class)))
                .thenReturn(new ArrayList<>());

        List<ReservationRecord> records = reservationService.getAllRecordsByArenaId(123L,
                Timestamp.valueOf(LocalDateTime.MIN), Timestamp.valueOf(LocalDateTime.MAX));
        assert records.size() == 0;
    }

    @Test
    public void testTimeTableCollisions() {
        Mockito.when(reservationRepository.timeCollisionExist(any(Long.class), any(Timestamp.class), any(Timestamp.class)))
                .thenReturn(10);
        assert reservationService.checkTimeTableCollision( 10L,
                Timestamp.valueOf(LocalDateTime.MIN), Timestamp.valueOf(LocalDateTime.MAX));

        Mockito.when(reservationRepository.timeCollisionExist(
                        15L,
                        Timestamp.valueOf(LocalDateTime.of(2026, Month.DECEMBER, 24, 10, 0)),
                        Timestamp.valueOf(LocalDateTime.of(2026, Month.DECEMBER, 24, 18, 0))))
                .thenReturn(0);

        assert !reservationService.checkTimeTableCollision(
                15L,
                Timestamp.valueOf(LocalDateTime.of(2026, Month.DECEMBER, 24, 10, 0)),
                Timestamp.valueOf(LocalDateTime.of(2026, Month.DECEMBER, 24, 18, 0)));
    }

    @Test
    public void testGetArenaWithExistingId() {
        Mockito.when(reservationRepository.findById(any(Long.class)))
                .thenReturn(Optional.ofNullable(reservationRecords.get(0)));
        ReservationRecord record = reservationService.findById(1L);

        assert record.equals(reservationRecords.get(0));
    }

    @Test
    public void testGetArenaWithNonExistingId() {
        Mockito.when(reservationRepository.findById(any(Long.class)))
                .thenReturn(Optional.empty());

        assertThrows(ResourceNotFoundException.class, () -> reservationService.findById(123L));
    }

    @Test
    public void testDeleteArena() {
        List<Long> ids = Arrays.asList(123L, 234L, 345L, 456L, 567L, 678L, 789L);
        for(Long id : ids) {
            reservationService.deleteRecord(id);
            Mockito.verify(reservationRepository).deleteById(id);
        }
    }

    @Test
    public void testDeleteAllArenas() {
        reservationService.deleteAll();
        Mockito.verify(reservationRepository).deleteAll();
    }
}
