package cz.muni.fi.icehockeymanager.discussionforum.rest;

import cz.muni.fi.icehockeymanager.discussionforum.api.dto.create.CreateCommentDto;
import cz.muni.fi.icehockeymanager.discussionforum.api.dto.view.CommentDto;
import cz.muni.fi.icehockeymanager.discussionforum.api.dto.view.UserDto;
import cz.muni.fi.icehockeymanager.discussionforum.facade.CommentFacade;
import cz.muni.fi.icehockeymanager.discussionforum.service.UserInfoService;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.security.SecurityRequirement;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.security.core.annotation.AuthenticationPrincipal;
import org.springframework.security.oauth2.jwt.Jwt;
import org.springframework.web.bind.annotation.*;

import java.util.List;

@RestController
@RequestMapping("/comments")
public class CommentRestController {
    private final CommentFacade commentFacade;
    private final UserInfoService userInfoService;

    @Autowired
    public CommentRestController(CommentFacade commentFacade, UserInfoService userInfoService) {
        this.commentFacade = commentFacade;
        this.userInfoService = userInfoService;
    }

    @Operation(summary = "List all comments")
    @GetMapping
    @ResponseStatus(HttpStatus.OK)
    public ResponseEntity<List<CommentDto>> listComments() {
        return ResponseEntity.ok(commentFacade.getAllComments());
    }

    @Operation(summary = "Get details about a specific comment")
    @GetMapping("/{commentId}")
    @ResponseStatus(HttpStatus.OK)
    public ResponseEntity<CommentDto> getComment(@PathVariable("commentId") Long commentId) {
        return ResponseEntity.ok(commentFacade.getCommentById(commentId));
    }

    @Operation(
        summary = "Create a new comment in a discussion thread",
        security = @SecurityRequirement(name = "Bearer")
    )
    @PostMapping
    @ResponseStatus(HttpStatus.CREATED)
    public ResponseEntity<CommentDto> createRootComment(@AuthenticationPrincipal Jwt principal, @RequestBody CreateCommentDto createCommentDto) {
        Long authorId = 0L;
        if (principal != null) {
            UserDto userInfo = userInfoService.getUserInfo(principal);
            authorId = userInfo.getId();
        }
        return new ResponseEntity<>(commentFacade.createRootComment(authorId, createCommentDto), HttpStatus.CREATED);
    }

    @Operation(
        summary = "Reply to a comment",
        security = @SecurityRequirement(name = "Bearer")
    )
    @PostMapping("/reply")
    @ResponseStatus(HttpStatus.CREATED)
    public ResponseEntity<CommentDto> createReplyComment(@AuthenticationPrincipal Jwt principal, @RequestBody CreateCommentDto createCommentDto) {
        Long authorId = 0L;
        if (principal != null) {
            UserDto userInfo = userInfoService.getUserInfo(principal);
            authorId = userInfo.getId();
        }
        return new ResponseEntity<>(commentFacade.createReplyComment(authorId, createCommentDto), HttpStatus.CREATED);
    }

    @Operation(
        summary = "Delete a comment",
        security = @SecurityRequirement(name = "Bearer")
    )
    @DeleteMapping("/{commentId}")
    @ResponseStatus(HttpStatus.NO_CONTENT)
    public ResponseEntity<Void> deleteComment(@PathVariable("commentId") Long commentId) {
        commentFacade.deleteComment(commentId);
        return ResponseEntity.noContent().build();
    }
}
