package cz.muni.fi.icehockeymanager.discussionforum.rest;

import cz.muni.fi.icehockeymanager.discussionforum.api.dto.create.CreateCommentDto;
import cz.muni.fi.icehockeymanager.discussionforum.api.dto.view.CommentDto;
import cz.muni.fi.icehockeymanager.discussionforum.facade.CommentFacade;
import cz.muni.fi.icehockeymanager.discussionforum.service.UserInfoService;
import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.Test;
import org.mockito.Mockito;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.WebMvcTest;
import org.springframework.boot.test.mock.mockito.MockBean;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.test.web.servlet.MvcResult;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.List;
import java.util.TimeZone;

import static org.mockito.ArgumentMatchers.any;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.*;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;

@WebMvcTest(controllers = CommentRestController.class)
public class CommentRestControllerTest {

    @Autowired
    private MockMvc mockMvc;

    @MockBean
    private CommentFacade commentFacade;
    @MockBean
    private UserInfoService userInfoService;

    @Test
    public void testListComments() throws Exception {
        List<CommentDto> comments = new ArrayList<>();
        comments.add(getDummyComment());

        Mockito.when(commentFacade.getAllComments()).thenReturn(comments);

        MvcResult response = mockMvc.perform(get("/comments"))
                .andExpect(status().isOk())
                .andReturn();

        Assertions.assertEquals(
                "[{\"id\":1,\"content\":\"Sample comment\"," +
                        "\"createdAt\":\"2023-03-20T22:00:00.000+00:00\"," +
                        "\"lastEditedAt\":\"2023-03-24T22:45:30.000+00:00\"," +
                        "\"authorId\":10,\"replies\":[]}]",
                response.getResponse().getContentAsString()
        );
    }

    @Test
    public void testCreateRootComment() throws Exception {

        Mockito.when(commentFacade.createRootComment(any(Long.class), any(CreateCommentDto.class)))
                .thenReturn(getDummyComment());

        MvcResult response = mockMvc.perform(post("/comments")
                        .contentType("application/json")
                        .content("{\"name\":\"Comment\"}"))
                .andExpect(status().isCreated())
                .andReturn();

        Assertions.assertEquals(
                "{\"id\":1,\"content\":\"Sample comment\"," +
                        "\"createdAt\":\"2023-03-20T22:00:00.000+00:00\"," +
                        "\"lastEditedAt\":\"2023-03-24T22:45:30.000+00:00\"," +
                        "\"authorId\":10,\"replies\":[]}",
                response.getResponse().getContentAsString()
        );
    }

    @Test
    public void testCreateReplyComment() throws Exception {

        Mockito.when(commentFacade.createReplyComment(any(Long.class), any(CreateCommentDto.class)))
                .thenReturn(getDummyComment());

        MvcResult response = mockMvc.perform(post("/comments/reply")
                        .contentType("application/json")
                        .content("{\"name\":\"Comment\"}"))
                .andExpect(status().isCreated())
                .andReturn();

        Assertions.assertEquals(
                "{\"id\":1,\"content\":\"Sample comment\"," +
                        "\"createdAt\":\"2023-03-20T22:00:00.000+00:00\"," +
                        "\"lastEditedAt\":\"2023-03-24T22:45:30.000+00:00\"," +
                        "\"authorId\":10,\"replies\":[]}",
                response.getResponse().getContentAsString()
        );
    }

    @Test
    public void testDeleteComment() throws Exception {
        mockMvc.perform(delete("/comments/1"))
                .andExpect(status().isNoContent());
        Mockito.verify(commentFacade).deleteComment(1L);
    }

    @Test
    public void testGetCommentById() throws Exception {
        Mockito.when(commentFacade.getCommentById(1L))
                .thenReturn(getDummyComment());

        MvcResult response = mockMvc.perform(get("/comments/1"))
                .andExpect(status().isOk())
                .andReturn();

        Assertions.assertEquals(
                "{\"id\":1,\"content\":\"Sample comment\"," +
                        "\"createdAt\":\"2023-03-20T22:00:00.000+00:00\"," +
                        "\"lastEditedAt\":\"2023-03-24T22:45:30.000+00:00\"," +
                        "\"authorId\":10,\"replies\":[]}",
                response.getResponse().getContentAsString()
        );
    }

    private static CommentDto getDummyComment() {
        CommentDto comment = new CommentDto();
        comment.setId(1L);
        comment.setContent("Sample comment");
        comment.setAuthorId(10L);
        Calendar calendar = Calendar.getInstance(TimeZone.getTimeZone("UTC"));
        calendar.clear();
        calendar.set(2023, Calendar.MARCH, 20, 22, 0, 0);
        comment.setCreatedAt(calendar.getTime());
        calendar.set(2023, Calendar.MARCH, 24, 22, 45, 30);
        comment.setLastEditedAt(calendar.getTime());
        comment.setReplies(new ArrayList<>());
        return comment;
    }
}
