package cz.muni.fi.icehockeymanager.discussionforum.service;

import cz.muni.fi.icehockeymanager.discussionforum.data.domain.Category;
import cz.muni.fi.icehockeymanager.discussionforum.data.domain.Comment;
import cz.muni.fi.icehockeymanager.discussionforum.data.domain.Thread;
import cz.muni.fi.icehockeymanager.discussionforum.data.repository.CommentRepository;
import cz.muni.fi.icehockeymanager.discussionforum.exceptions.ResourceNotFoundException;
import org.junit.jupiter.api.BeforeAll;
import org.junit.jupiter.api.Test;
import org.mockito.Mockito;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.boot.test.mock.mockito.MockBean;

import java.util.*;

import static org.mockito.ArgumentMatchers.any;

@SpringBootTest
public class CommentServiceTest {
    @Autowired
    private CommentService commentService;

    @MockBean
    private CommentRepository commentRepository;
    private static final Category category = new Category(1L, "Ice Hockey World Championship 2023", new ArrayList<>());
    private static final List<Thread> threads = new ArrayList<>();
    private static final List<Comment> comments = new ArrayList<>();

    @BeforeAll
    public static void initializeBeforeTests() {
        threads.add(new Thread(4L, "IIHF 2023 Winner", "Who will win Ice Hockey World Championship 2023",
                new Date(), new Date(), 1L, category, new ArrayList<>()));
        Comment rootComment1 = new Comment(1L, "I am not an expert, but Canada looking sharp as ever!",
                new Date(), new Date(), 123L, threads.get(0), null, null);
        Comment reply1ToRoot1 = new Comment(2L, "What? No, no, no, USA will definitely win this time",
                new Date(), new Date(), 124L, threads.get(0), rootComment1, null);
        Comment reply2ToRoot1 = new Comment(3L, "I agree, though they need to watch out for the Czechs",
                new Date(), new Date(), 125L, threads.get(0), rootComment1, null);
        Comment reply3toReply2 = new Comment(4L, "Jaromir Jagr will shown them how to play #goodOldDays",
                new Date(), new Date(), 126L, threads.get(0), reply2ToRoot1, null);
        Comment reply4toReply2 = new Comment(5L, "Dont be silly my friend, the only problem will be the Finns",
                new Date(), new Date(), 127L, threads.get(0), reply2ToRoot1, null);
        rootComment1.setReplies(Arrays.asList(reply1ToRoot1, reply2ToRoot1));
        reply2ToRoot1.setReplies(Arrays.asList(reply3toReply2, reply4toReply2));
        comments.addAll(Arrays.asList(rootComment1, reply1ToRoot1, reply2ToRoot1, reply3toReply2, reply4toReply2));


        threads.add(new Thread(10L, "Ice Hockey Veterans", "Veterans and best players of IIHF matches",
                new Date(), new Date(), 1L, category, new ArrayList<>()));
        Comment rootComment2 = new Comment(10L, "Well, Verstappen and Hamilton probably were the best",
                new Date(), new Date(), 200L, threads.get(1), null, null);
        Comment reply1ToRoot2 = new Comment(11L, "Wrong forum my F1 brother :/",
                new Date(), new Date(), 201L, threads.get(1), rootComment2, null);
        Comment reply2ToRoot2 = new Comment(12L, "Get this guy out of here, he is always making fun of ice hockey",
                new Date(), new Date(), 201L, threads.get(1), rootComment2, null);
        rootComment2.setReplies(Arrays.asList(reply1ToRoot2, reply2ToRoot2));
        comments.addAll(Arrays.asList(rootComment2, reply1ToRoot2, reply2ToRoot2));

        Comment rootComment3 = new Comment(20L, "TBH Satan, Jagr and Ovekchin and in this order",
                new Date(), new Date(), 300L, threads.get(1), null, null);
        Comment reply1ToRoot3 = new Comment(21L, "True, do not mess with Satan",
                new Date(), new Date(), 301L, threads.get(1), rootComment3, null);
        rootComment3.setReplies(Arrays.asList(reply1ToRoot3));
        comments.addAll(Arrays.asList(rootComment3, reply1ToRoot3));
    }

    @Test
    void testGetExistingRootComment() {
        Mockito.when(commentRepository.findById(1L))
                .thenReturn(Optional.ofNullable(comments.get(0)));
        Comment comment = commentService.getCommentById(1L);
        assert comment.equals(comments.get(0));
        assert comment.getParentComment() == null;
        assert !comment.getReplies().isEmpty();
    }

    @Test
    public void testGetExistingCommentWithId() {
        Mockito.when(commentRepository.findById(4L))
                .thenReturn(Optional.ofNullable(comments.get(3)));
        Comment comment = commentService.getCommentById(4L);
        assert comment.equals(comments.get(3));
        assert !comment.equals(comments.get(0));
        assert comment.getParentComment().equals(comments.get(2));
    }

    @Test
    public void testGetNonExistingCommentWithId() {
        Mockito.when(commentRepository.findById(any(Long.class)))
                .thenReturn(Optional.empty());
        try {
            commentService.getCommentById(123L);
            assert false;
        } catch (ResourceNotFoundException resourceNotFoundException) {
            assert true;
        }
    }

    @Test
    public void testGetAllComments() {
        Mockito.when(commentRepository.findAll())
                .thenReturn(comments);
        List<Comment> commentList = commentService.getAllComments();
        assert Objects.equals(commentList.size(), comments.size());

        int index = 0;
        for (Comment comment : commentList) {
            assert comment.equals(comments.get(index));
            index += 1;
        }
    }

    @Test
    public void testGetAllEmptyListOfComments() {
        Mockito.when(commentRepository.findAll())
                .thenReturn(new ArrayList<>());
        List<Comment> commentList = commentService.getAllComments();
        assert commentList != null;
        assert commentList.isEmpty();
    }

    @Test
    public void testCreateComment() {
        Mockito.when(commentRepository.save(any(Comment.class)))
                .thenReturn(comments.get(0));
        Comment newComment = commentService.createComment(comments.get(0));
        assert newComment.equals(comments.get(0));
        assert !newComment.equals(comments.get(1));
    }

    @Test
    public void testDeleteComment() {
        List<Long> ids = Arrays.asList(1L, 2L, 3L, 4L, 5L, 6L);
        for (Long id : ids) {
            commentService.deleteComment(id);
            Mockito.verify(commentRepository).deleteById(id);
        }
    }
}
