package cz.muni.fi.icehockeymanager.discussionforum.service;

import cz.muni.fi.icehockeymanager.discussionforum.data.domain.Category;
import cz.muni.fi.icehockeymanager.discussionforum.data.domain.Thread;
import cz.muni.fi.icehockeymanager.discussionforum.data.repository.ThreadRepository;
import cz.muni.fi.icehockeymanager.discussionforum.exceptions.ResourceNotFoundException;
import org.junit.jupiter.api.BeforeAll;
import org.junit.jupiter.api.Test;
import org.mockito.Mockito;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.boot.test.mock.mockito.MockBean;

import java.util.*;

import static org.mockito.ArgumentMatchers.any;

@SpringBootTest
public class ThreadServiceTest {

    @Autowired
    ThreadService threadService;

    @MockBean
    ThreadRepository threadRepository;

    private static final List<Category> categories = Arrays.asList(
            new Category(123L, "Newbies at World Championship 2023", new ArrayList<>()),
            new Category(124L, "Ice Hockey World Championship 2023", new ArrayList<>())
    );
    private static final List<Thread> threads = new ArrayList<>();

    @BeforeAll
    public static void initializeBeforeTests() {
        threads.add(new Thread(1L, "Newbies from Europe", "Newbies from Europe to look out for",
                new Date(), new Date(), 1L, categories.get(0), new ArrayList<>()));
        threads.add(new Thread(2L, "Newbies from America", "Newbies from America to look out for",
                new Date(), new Date(), 1L, categories.get(0), new ArrayList<>()));
        threads.add(new Thread(3L, "Russia at IIHF 2023", "Should russian players participate at IIHF 2023",
                new Date(), new Date(), 10L, categories.get(1), new ArrayList<>()));
        threads.add(new Thread(4L, "IIHF 2023 Winner", "Who will win Ice Hockey World Championship 2023",
                new Date(), new Date(), 1L, categories.get(1), new ArrayList<>()));
    }

    private boolean compareThreads(Thread first, Thread second) {
        first.setCreatedAt(second.getCreatedAt());
        first.setLastEditedAt(second.getLastEditedAt());
        return first.equals(second);
    }

    @Test
    public void testGetExistingThreadWithId() {
        Mockito.when(threadRepository.findById(2L))
                .thenReturn(Optional.ofNullable(threads.get(1)));
        Thread thread = threadService.getThreadById(2L);
        assert compareThreads(thread, threads.get(1));
    }

    @Test
    public void testGetNonExistingThreadWithId() {
        Mockito.when(threadRepository.findById(any(Long.class)))
                .thenReturn(Optional.empty());
        try {
            threadService.getThreadById(123L);
            assert false;
        } catch (ResourceNotFoundException resourceNotFoundException) {
            assert true;
        }
    }

    @Test
    public void testGetAllThreads() {
        Mockito.when(threadRepository.findAll())
                .thenReturn(threads);
        List<Thread> threadsList = threadService.getAllThreads();
        assert Objects.equals(threadsList.size(), threads.size());

        int index = 0;
        for (Thread thread : threadsList) {
            assert compareThreads(thread, threads.get(index));
            index += 1;
        }
    }

    @Test
    public void testGetAllEmptyListOfThreads() {
        Mockito.when(threadRepository.findAll())
                .thenReturn(new ArrayList<>());
        List<Thread> threadsList = threadService.getAllThreads();
        assert threadsList != null;
        assert threadsList.isEmpty();
    }

    @Test
    public void testCreateThread() {
        Mockito.when(threadRepository.save(any(Thread.class)))
                .thenReturn(threads.get(3));
        Thread newThread = threadService.createThread(threads.get(3));
        assert compareThreads(newThread, threads.get(3));
        assert !compareThreads(newThread, threads.get(0));
    }

    @Test
    public void testDeleteThread() {
        List<Long> ids = Arrays.asList(1L, 2L, 3L, 4L, 5L, 6L);
        for (Long id : ids) {
            threadService.deleteThread(id);
            Mockito.verify(threadRepository).deleteById(id);
        }
    }
}
