package cz.muni.fi.icehockeymanager.core.service;

import cz.muni.fi.icehockeymanager.core.data.domain.Game;
import cz.muni.fi.icehockeymanager.core.data.domain.Team;
import cz.muni.fi.icehockeymanager.core.data.enums.GameStatus;
import cz.muni.fi.icehockeymanager.core.data.repository.GameRepository;
import cz.muni.fi.icehockeymanager.core.exceptions.ResourceNotFoundException;
import cz.muni.fi.icehockeymanager.core.exceptions.TeamCompositionException;
import jakarta.transaction.Transactional;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;

@Service
@Transactional
public class GameService {

    private final GameRepository gameRepository;

    @Autowired
    public GameService(GameRepository gameRepository) {
        this.gameRepository = gameRepository;
    }

    public List<Game> getAllGames() {
        return gameRepository.findAll();
    }

    public Game createGame(Game game) {
        return gameRepository.save(game);
    }

    public Game getGameById(Long gameId) {
        return gameRepository.findById(gameId).orElseThrow(
                () -> new ResourceNotFoundException("Game with id " + gameId + " not found."));
    }

    public void deleteGame(Long gameId) {
        if (gameRepository.deleteGameById(gameId) != 1) {
            throw new ResourceNotFoundException("Game with id " + gameId + " does not exist");
        }
    }

    public Game startGame(Long gameId) {
        Game game = getGameById(gameId);
        verifyTeamComposition(game.getHomeTeam());
        verifyTeamComposition(game.getAwayTeam());
        startGame(game);
        // TODO: process results and notify subscribers via ReportService
        return gameRepository.save(game);
    }

    private void verifyTeamComposition(Team team) {
        if (!team.hasGoalkeeper()) {
            throw new TeamCompositionException("Team " + team.getName() + " has no goalkeeper.");
        }
        if (team.getPlayers().size() < 6) {
            // at least 6 players: goalkeeper + one formation
            throw new TeamCompositionException("Team " + team.getName() + " doesn't have enough players.");
        }
    }

    private void startGame(Game game) {
        game.setStatus(GameStatus.IN_PROGRESS);
        double homeTeamStrength = game.getHomeTeam().getStrength();
        double awayTeamStrength = game.getAwayTeam().getStrength();
        game.setHomeTeamGoals(generateNumberOfGoals(homeTeamStrength, true));
        game.setAwayTeamGoals(generateNumberOfGoals(awayTeamStrength, false));
        game.setStatus(GameStatus.FINISHED);
    }

    private Integer generateNumberOfGoals(double strength, boolean isHomeTeam) {
        // strength is 0-100
        double luck = -10 + Math.random() * 20;
        int morale = isHomeTeam? 5 : 0;
        return (int)((Math.random() * (strength + luck + morale)) / 10);
    }
}
