package cz.muni.fi.icehockeymanager.core.rest;

import cz.muni.fi.icehockeymanager.core.api.create.CreateLeagueDto;
import cz.muni.fi.icehockeymanager.core.api.view.LeagueDetailDto;
import cz.muni.fi.icehockeymanager.core.api.view.LeagueDto;
import cz.muni.fi.icehockeymanager.core.api.view.TeamDto;
import cz.muni.fi.icehockeymanager.core.facade.LeagueFacade;
import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.Test;
import org.mockito.Mockito;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.WebMvcTest;
import org.springframework.boot.test.mock.mockito.MockBean;
import org.springframework.security.oauth2.client.registration.ClientRegistrationRepository;
import org.springframework.security.test.context.support.WithMockUser;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.test.web.servlet.MvcResult;

import java.util.ArrayList;
import java.util.List;

import static org.mockito.ArgumentMatchers.any;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.*;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;

@WebMvcTest(controllers = LeagueRestController.class)
@WithMockUser(value = "spring", authorities = "ADMIN")
public class LeagueRestControllerTest {

    @Autowired
    private MockMvc mockMvc;

    @MockBean
    private LeagueFacade leagueFacade;

    @MockBean
    private ClientRegistrationRepository clientRegistrationRepository;

    @Test
    public void testListLeagues() throws Exception {
        LeagueDto league = new LeagueDto();
        league.setId(1L);
        league.setName("League");
        league.setAcronym("L");

        List<LeagueDto> leagues = new ArrayList<>();
        leagues.add(league);

        Mockito.when(leagueFacade.getAllLeagues()).thenReturn(leagues);

        MvcResult response = mockMvc.perform(get("/leagues"))
                .andExpect(status().isOk())
                .andReturn();

        Assertions.assertEquals(
                "[{\"id\":1,\"name\":\"League\",\"acronym\":\"L\"}]",
                response.getResponse().getContentAsString()
        );
    }

    @Test
    public void testCreateLeague() throws Exception {
        LeagueDto leagueDto = new LeagueDto();
        leagueDto.setId(1L);
        leagueDto.setName("League");
        leagueDto.setAcronym("L");

        Mockito.when(leagueFacade.createLeague(any(CreateLeagueDto.class))).thenReturn(leagueDto);

        MvcResult response = mockMvc.perform(post("/leagues")
                .contentType("application/json")
                .content("{\"name\":\"League\",\"acronym\":\"L\"}"))
                .andExpect(status().isCreated())
                .andReturn();

        Assertions.assertEquals(
                "{\"id\":1,\"name\":\"League\",\"acronym\":\"L\"}",
                response.getResponse().getContentAsString()
        );
    }

    @Test
    public void testDeleteLeague() throws Exception {
        mockMvc.perform(delete("/leagues/1"))
                .andExpect(status().isNoContent());
        Mockito.verify(leagueFacade).deleteLeague(1L);
    }

    @Test
    public void testGetLeagueById() throws Exception {
        TeamDto team = new TeamDto();
        team.setId(1L);
        team.setName("Team");
        team.setCreatedById(1L);

        LeagueDetailDto leagueDto = new LeagueDetailDto();
        leagueDto.setId(1L);
        leagueDto.setName("League");
        leagueDto.setAcronym("L");
        leagueDto.addTeam(team);

        Mockito.when(leagueFacade.getLeagueById(1L)).thenReturn(leagueDto);

        MvcResult response = mockMvc.perform(get("/leagues/1"))
                .andExpect(status().isOk())
                .andReturn();

        Assertions.assertEquals(
                "{\"id\":1,\"name\":\"League\",\"acronym\":\"L\",\"teams\":[{\"id\":1,\"name\":\"Team\",\"createdById\":1}],\"schedules\":[]}",
                response.getResponse().getContentAsString()
        );
    }
}
