package cz.muni.fi.icehockeymanager.core.rest;

import cz.muni.fi.icehockeymanager.core.api.create.CreateTeamDto;
import cz.muni.fi.icehockeymanager.core.api.view.TeamDetailDto;
import cz.muni.fi.icehockeymanager.core.api.view.TeamDto;
import cz.muni.fi.icehockeymanager.core.api.view.UserDto;
import cz.muni.fi.icehockeymanager.core.facade.TeamFacade;
import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.Test;
import org.mockito.Mockito;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.WebMvcTest;
import org.springframework.boot.test.mock.mockito.MockBean;
import org.springframework.security.oauth2.client.registration.ClientRegistrationRepository;
import org.springframework.security.test.context.support.WithMockUser;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.test.web.servlet.MvcResult;

import java.util.ArrayList;
import java.util.List;
import java.util.Optional;

import static org.mockito.ArgumentMatchers.any;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.*;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;

@WebMvcTest(controllers = TeamRestController.class)
@WithMockUser(value = "spring")
public class TeamRestControllerTest {

    @Autowired
    private MockMvc mockMvc;

    @MockBean
    private TeamFacade teamFacade;

    @MockBean
    private ClientRegistrationRepository clientRegistrationRepository;

    @Test
    public void testListTeams() throws Exception {
        TeamDto team = new TeamDto();
        team.setId(1L);
        team.setName("Team");
        team.setCreatedById(1L);

        List<TeamDto> teams = new ArrayList<>();
        teams.add(team);

        Mockito.when(teamFacade.getAllTeams(any(Optional.empty().getClass()))).thenReturn(teams);

        MvcResult response = mockMvc.perform(get("/teams"))
                .andExpect(status().isOk())
                .andReturn();

        Assertions.assertEquals(
                "[{\"id\":1,\"name\":\"Team\",\"createdById\":1}]",
                response.getResponse().getContentAsString()
        );
    }

    @Test
    public void testListTeamsWithFilter() throws Exception {
        TeamDto team = new TeamDto();
        team.setId(1L);
        team.setName("Team");
        team.setCreatedById(1L);

        List<TeamDto> teams = new ArrayList<>();
        teams.add(team);

        Mockito.when(teamFacade.getAllTeams(any(Optional.of(1L).getClass()))).thenReturn(teams);

        MvcResult response = mockMvc.perform(get("/teams?createdById=1"))
                .andExpect(status().isOk())
                .andReturn();

        Assertions.assertEquals(
                "[{\"id\":1,\"name\":\"Team\",\"createdById\":1}]",
                response.getResponse().getContentAsString()
        );
    }

    @Test
    public void testCreateTeam() throws Exception {
        TeamDetailDto teamDetailDto = new TeamDetailDto();
        teamDetailDto.setId(1L);
        teamDetailDto.setName("Team");
        UserDto user = new UserDto(3L, "name", "surname", "mail", false);
        teamDetailDto.setCreatedBy(user);

        Mockito.when(teamFacade.createTeam(any(CreateTeamDto.class))).thenReturn(teamDetailDto);

        MvcResult response = mockMvc.perform(post("/teams")
                .contentType("application/json")
                .content("{\"name\":\"Team\",\"createdById\":1}"))
                .andExpect(status().isCreated())
                .andReturn();

        Assertions.assertEquals(
                "{\"id\":1,\"name\":\"Team\",\"createdBy\":" +
                        "{\"id\":3,\"firstName\":\"name\",\"lastName\":\"surname\",\"email\":\"mail\",\"isAdmin\":false}," +
                        "\"players\":[]}",
                response.getResponse().getContentAsString()
        );
    }

    @Test
    public void testGetTeam() throws Exception {
        TeamDetailDto teamDetailDto = new TeamDetailDto();
        teamDetailDto.setId(1L);
        teamDetailDto.setName("Team");
        UserDto user = new UserDto(3L, "name", "surname", "mail", false);
        teamDetailDto.setCreatedBy(user);

        Mockito.when(teamFacade.getTeamById(1L)).thenReturn(teamDetailDto);

        MvcResult response = mockMvc.perform(get("/teams/1"))
                .andExpect(status().isOk())
                .andReturn();

        Assertions.assertEquals(
                "{\"id\":1,\"name\":\"Team\",\"createdBy\":" +
                        "{\"id\":3,\"firstName\":\"name\",\"lastName\":\"surname\",\"email\":\"mail\",\"isAdmin\":false}," +
                        "\"players\":[]}",
                response.getResponse().getContentAsString()
        );
    }

    @Test
    public void testDeleteTeam() throws Exception {
        mockMvc.perform(delete("/teams/1"))
                .andExpect(status().isNoContent());
        Mockito.verify(teamFacade).deleteTeam(1L);
    }

    @Test
    public void testAddPlayer() throws Exception {
        mockMvc.perform(patch("/teams/1/players/1"))
                .andExpect(status().isOk());
        Mockito.verify(teamFacade).addPlayer(1L, 1L);
    }
}
