package cz.muni.fi.icehockeymanager.report;

import cz.muni.fi.icehockeymanager.report.api.create.CreateSubscriberDto;
import cz.muni.fi.icehockeymanager.report.api.view.SubscriberDto;
import cz.muni.fi.icehockeymanager.report.data.enums.SubscribeSubject;
import cz.muni.fi.icehockeymanager.report.facade.SubscriberFacade;
import cz.muni.fi.icehockeymanager.report.rest.SubscriberRestController;
import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.Test;
import org.mockito.Mockito;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.WebMvcTest;
import org.springframework.boot.test.mock.mockito.MockBean;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.test.web.servlet.MvcResult;
import org.springframework.test.web.servlet.request.MockMvcRequestBuilders;

import static org.mockito.ArgumentMatchers.any;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.*;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;

import java.util.List;


@WebMvcTest(controllers = {SubscriberRestController.class})
public class SubscriberRestControllerTest {

    @Autowired
    private MockMvc mockMvc;

    @MockBean
    private SubscriberFacade subscriberFacade;

    private final String CREATE_SUBSCRIBER_JSON = "{\"name\":\"Adam\",\"mail\":\"adam@gmail.com\",\"playerRefId\":1,\"subscribedSubjects\":[\"GAME_CANCELLED\"]}";
    private final String SUBSCRIBER_1_JSON = "{\"id\":1,\"name\":\"John Doe\",\"mail\":\"john.doe@gmail.com\",\"playerRefId\":12345,\"subscribedSubjects\":[\"GAME_CANCELLED\",\"GAME_RESCHEDULED\"]}";
    private final String SUBSCRIBER_2_JSON = "{\"id\":2,\"name\":\"Jane Smith\",\"mail\":\"jane.smith@mail.muni.cz\",\"playerRefId\":67890,\"subscribedSubjects\":[\"NEW_GAME_SCHEDULED\"]}";
    private final String NONEXISTENT_SUBSCRIBER_JSON = "{\"id\":3,\"name\":\"Alice Johnson\",\"mail\":\"alice.johnson@mail.muni.cz\",\"playerRefId\":132,\"subscribedSubjects\":[\"GAME_COMPLETED\",\"GAME_CANCELLED\"]}";
    private final String SUBSCRIBERS_JSON = "[" + SUBSCRIBER_1_JSON + "," + SUBSCRIBER_2_JSON + "]";
    private final SubscriberDto subscriber1Dto;
    private final SubscriberDto subscriber2Dto;
    private final SubscriberDto nonexistentSubscriberDto;

    public SubscriberRestControllerTest() {
        subscriber1Dto = new SubscriberDto();
        subscriber2Dto = new SubscriberDto();
        nonexistentSubscriberDto = new SubscriberDto();

        subscriber1Dto.setId(1L);
        subscriber1Dto.setName("John Doe");
        subscriber1Dto.setMail("john.doe@gmail.com");
        subscriber1Dto.setPlayerRefId(12345L);
        subscriber1Dto.setSubscribedSubjects(List.of(SubscribeSubject.GAME_CANCELLED, SubscribeSubject.GAME_RESCHEDULED));

        subscriber2Dto.setId(2L);
        subscriber2Dto.setName("Jane Smith");
        subscriber2Dto.setMail("jane.smith@mail.muni.cz");
        subscriber2Dto.setPlayerRefId(67890L);
        subscriber2Dto.setSubscribedSubjects(List.of(SubscribeSubject.NEW_GAME_SCHEDULED));

        nonexistentSubscriberDto.setId(3L);
        nonexistentSubscriberDto.setName("Alice Johnson");
        nonexistentSubscriberDto.setMail("alice.johnson@mail.muni.cz");
        nonexistentSubscriberDto.setPlayerRefId(132L);
        nonexistentSubscriberDto.setSubscribedSubjects(List.of(SubscribeSubject.GAME_COMPLETED, SubscribeSubject.GAME_CANCELLED));
    }

    @Test
    public void testGetSubscriber() throws Exception {
        // subscriber 1
        Mockito.when(subscriberFacade.getSubscriberById(1L)).thenReturn(subscriber1Dto);

        MvcResult response = mockMvc.perform(MockMvcRequestBuilders.get("/subscribers/1"))
                                    .andExpect(status().isOk())
                                    .andReturn();

        Assertions.assertEquals(SUBSCRIBER_1_JSON, response.getResponse().getContentAsString());

        // subscriber 2
        Mockito.when(subscriberFacade.getSubscriberById(2L)).thenReturn(subscriber2Dto);

        response = mockMvc.perform(MockMvcRequestBuilders.get("/subscribers/2"))
                .andExpect(status().isOk())
                .andReturn();

        Assertions.assertEquals(SUBSCRIBER_2_JSON, response.getResponse().getContentAsString());
    }

    @Test
    public void testGetSubscriberFail() throws Exception {
        Mockito.when(subscriberFacade.getSubscriberById(100L)).thenReturn(null);

        mockMvc.perform(MockMvcRequestBuilders.get("/subscribers/100"))
                .andExpect(status().isNotFound());
    }

    @Test
    public void testGetSubscribers() throws Exception {
        Mockito.when(subscriberFacade.getAllSubscribers()).thenReturn(List.of(subscriber1Dto, subscriber2Dto));

        MvcResult result = mockMvc.perform(MockMvcRequestBuilders.get("/subscribers"))
                .andExpect(status().isOk())
                .andReturn();

        Assertions.assertEquals(SUBSCRIBERS_JSON, result.getResponse().getContentAsString());
    }

    @Test
    public void testCreateSubscriber() throws Exception {
        Mockito.when(subscriberFacade.createSubscriber(any(CreateSubscriberDto.class))).thenReturn(nonexistentSubscriberDto);

        MvcResult response = mockMvc.perform(post("/subscribers")
                                .contentType("application/json")
                                .content(CREATE_SUBSCRIBER_JSON))
                                    .andExpect(status().isCreated())
                                    .andReturn();

        Assertions.assertEquals(NONEXISTENT_SUBSCRIBER_JSON, response.getResponse().getContentAsString());
    }

    @Test
    public void testDeleteSubscriber() throws Exception {
        Mockito.when(subscriberFacade.getSubscriberById(1L)).thenReturn(subscriber1Dto);

        mockMvc.perform(delete("/subscribers/1"))
                .andExpect(status().isNoContent());
        Mockito.verify(subscriberFacade).deleteSubscriber(1L);
    }

    @Test
    public void testDeleteNonexistentSubscriber() throws Exception {
        Mockito.when(subscriberFacade.getSubscriberById(100L)).thenReturn(null);

        mockMvc.perform(delete("/subscribers/100"))
                .andExpect(status().isNotFound());
    }

    @Test
    public void testUpdateSubscriber() throws Exception {
        Mockito.when(subscriberFacade.getSubscriberById(1L)).thenReturn(subscriber1Dto);

        mockMvc.perform(put("/subscribers/1")
                        .contentType("application/json")
                        .content(SUBSCRIBER_1_JSON)
                )
                .andExpect(status().isOk());

        Mockito.verify(subscriberFacade).updateSubscriber(1L, subscriber1Dto);
    }

    @Test
    public void testUpdateSubscriberFail() throws Exception {
        Mockito.when(subscriberFacade.getSubscriberById(100L)).thenReturn(null);

        mockMvc.perform(put("/subscribers/100")
                        .contentType("application/json")
                        .content(SUBSCRIBER_1_JSON)
                )
                .andExpect(status().isNotFound());
    }
}